=== modified file 'bzrlib/plugins/launchpad/lp_indirect.py'
--- bzrlib/plugins/launchpad/lp_indirect.py	2007-10-31 19:47:45 +0000
+++ bzrlib/plugins/launchpad/lp_indirect.py	2007-12-05 01:36:54 +0000
@@ -62,6 +62,17 @@
         if netloc != '':
             raise errors.InvalidURL(path=base)
 
+    def _requires_launchpad_login(self, scheme, netloc, path, query,
+                                  fragment):
+        """Does the URL require a Launchpad login in order to be reached?
+
+        The URL is specified by its parsed components, as returned from
+        urlsplit.
+        """
+        return (scheme in ('bzr+ssh', 'sftp')
+                and (netloc.endswith('launchpad.net')
+                     or netloc.endswith('launchpad.dev')))
+
     def _resolve(self, abspath,
                  _request_factory=ResolveLaunchpadPathRequest,
                  _lp_login=None):
@@ -83,8 +94,8 @@
             _lp_login = get_lp_login()
         for url in result['urls']:
             scheme, netloc, path, query, fragment = urlsplit(url)
-            if scheme == 'bzr+ssh' and (netloc.endswith('launchpad.net') or
-                                        netloc.endswith('launchpad.dev')):
+            if self._requires_launchpad_login(scheme, netloc, path, query,
+                                              fragment):
                 # Only accept launchpad.net bzr+ssh URLs if we know
                 # the user's Launchpad login:
                 if _lp_login is None:

=== modified file 'bzrlib/plugins/launchpad/test_lp_indirect.py'
--- bzrlib/plugins/launchpad/test_lp_indirect.py	2007-10-31 23:23:30 +0000
+++ bzrlib/plugins/launchpad/test_lp_indirect.py	2007-11-27 01:01:01 +0000
@@ -106,6 +106,18 @@
         self.assertEquals('http://bazaar.launchpad.net/~apt/apt/devel',
                           transport._resolve('lp:///apt', factory))
 
+    def test_skip_sftp_launchpad_net_when_anonymous(self):
+        # Test that sftp://bazaar.launchpad.net gets skipped if
+        # Bazaar does not know the user's Launchpad ID:
+        self.assertEqual(None, get_lp_login())
+        factory = FakeResolveFactory(
+            self, 'apt', dict(urls=[
+                    'sftp://bazaar.launchpad.net/~apt/apt/devel',
+                    'http://bazaar.launchpad.net/~apt/apt/devel']))
+        transport = LaunchpadTransport('lp:///')
+        self.assertEquals('http://bazaar.launchpad.net/~apt/apt/devel',
+                          transport._resolve('lp:///apt', factory))
+
     def test_rewrite_bzr_ssh_launchpad_net(self):
         # Test that bzr+ssh URLs get rewritten to include the user's
         # Launchpad ID (assuming we know the Launchpad ID).

